@@ -2,10 +2,11 @@
 #' @param x_filename Character. The filename of the input binary file.
 #' @param reference_raster Raster*. A Raster* object containing the header information to be used.
 #' @param out_nlayers Numeric. The number of layers in the flat binary file (defaults to nlayers(reference_raster)).
-#' @param dataType Character. The dataType of the flat binary file.  See ?dataType for available datatypes.  Default is 'FLT8S'.
+#' @param datatype Character. The dataType of the flat binary file.  See ?dataType for available datatypes.  Default is 'FLT8S'.
 #' @param bandorder Character. The bandorder ('BIP','BIL','BSQ') of the file. Default is 'BSQ'.
 #' @param format Character. The format of the header.  See ?hdr for valid entries.  Default is 'raster'.  CURRENTLY UNSUPPORTED.
 #' @param setMinMax Logical. Set the min/max for the file (will take longer to execute)?  Default=FALSE.
+#' @param additional_header Character. Create additional output headers for use with other GIS systems (see \code{\link{hdr}}). Set to NULL (default) to suppress.
 #' @param verbose logical. Enable verbose execution? Default is FALSE.  
 #' @author Jonathan A. Greenberg and Robert Hijimans (\email{spatial.tools@@estarcion.net})
 #' @seealso \code{\link{hdr}},\code{\link{dataType}}
@@ -13,17 +14,18 @@
 #' tahoe_highrez <- brick(system.file("external/tahoe_highrez.tif", package="spatial.tools"))
 #' test_blank_file <- create_blank_raster(filename=paste(tempfile(),".gri",sep=""),
 #' 	reference_raster=tahoe_highrez,nlayers=2,
-#' 	create_header=FALSE,format="raster",dataType="FLT8S",bandorder="BSQ")
+#' 	create_header=FALSE,format="raster",datatype="FLT8S",bandorder="BSQ")
 #' test_blank_raster <- build_raster_header(x_filename=test_blank_file,
 #' 	reference_raster=tahoe_highrez,out_nlayers=2,
-#' 	dataType='FLT8S',format='raster',bandorder="BSQ",setMinMax=TRUE)
+#' 	datatype='FLT8S',format='raster',bandorder="BSQ",setMinMax=TRUE)
 #' test_blank_raster
 #' }
 #' @import raster
 #' @export
 
 build_raster_header <- function(x_filename,reference_raster,out_nlayers,
-		dataType='FLT8S',format='raster',bandorder="BSQ",setMinMax=FALSE,
+		datatype='FLT8S',format='raster',bandorder="BSQ",setMinMax=FALSE,
+		additional_header=NULL,
 		verbose=FALSE)
 {
 #	require("raster")
@@ -41,7 +43,7 @@ build_raster_header <- function(x_filename,reference_raster,out_nlayers,
 	}
 	
 	outraster@file@name <- x_filename
-	outraster@file@datanotation <- dataType
+	outraster@file@datanotation <- datatype
 	outraster@file@bandorder <- bandorder
 	if(setMinMax) outraster@data@haveminmax <- TRUE	
 	else outraster@data@haveminmax <- FALSE
@@ -58,5 +60,11 @@ build_raster_header <- function(x_filename,reference_raster,out_nlayers,
 	
 	if(setMinMax) outraster <- setMinMax(outraster)
 	else outraster@data@haveminmax <- FALSE
+	
+	if(!is.null(additional_header))
+	{
+		hdr(outraster,format=additional_header)
+	}
+	
 	return(outraster)
 }
\ No newline at end of file
